import json
import threading
import uuid
from typing import Dict, Any, List

import requests

import dwai
from dwai.bailian.model_api.models import ChatQaMessage
_thread_context = threading.local()


class CompletionsRequestError(Exception):
    def __init__(self,
                 message=None,
                 status_code=None,
                 response_body=None
                 ):
        super(CompletionsRequestError, self).__init__(message)

        self.message = message
        self.status_code = status_code
        self.response_body = response_body

    def __str__(self):
        return "%s: status_code=%s, response_body=%s" % (
            self.message,
            self.status_code,
            self.response_body
        )

    def __repr__(self):
        return "(message=%r, status_code=%r, response_body=%r)" % (
            self.message,
            self.status_code,
            self.response_body
        )


class BaseCompletions:
    """ 调用百联进行文本生成 """

    @classmethod
    def call(cls, *args, **kwargs) -> Any:
        raise NotImplementedError()

    def reqeust(self, app_id: str,
                prompt: str,
                request_id: str = None,
                session_id: str = None,
                history: List[ChatQaMessage] = None,
                top_p: float = 0.0,
                biz_params: Dict = None,
                has_thoughts: bool = False,
                stream: bool = False,
                doc_reference_type: str = None):

        headers = dict()
        headers["Content-Type"] = "application/json;charset=UTF-8"
        headers["Authorization"] = "Bearer %s" % dwai.api_key

        if stream:
            headers["Accept"] = "text/event-stream"

        self.validate(app_id=app_id, prompt=prompt)

        if request_id is None:
            uuid_obj = uuid.uuid4()
            request_id = str(uuid_obj).replace('-', '')

        h = None
        if history is not None:
            h = []
            for v in history:
                h.append(v.to_dict())

        data = {
            "model": "qwen-plus-v1",
            "input": {
                "prompt": prompt,
                "history": []
            },
            "parameters": {
            }
        }

        url = "%s%s" % (dwai.api_base, "/bailian/services/aigc/text-generation/generation")
        session = self.__get_session()
        resp = session.request("POST",
                               url=url,
                               headers=headers,
                               data=json.dumps(data),
                               stream=stream)

        if not resp.ok:
            raise CompletionsRequestError("completion request error", resp.status_code, resp.text)

        return resp

    @staticmethod
    def __get_session() -> requests.Session:
        if not hasattr(_thread_context, "session"):
            _thread_context.session = requests.Session()

        return _thread_context.session

    @staticmethod
    def validate(app_id: str,
                 prompt: str):
        if dwai.api_base is None:
            dwai.api_base = "https://bailian.aliyuncs.com"

        if dwai.api_key is None:
            raise ValueError("you need to set DW_API_KEY before calling api")

        if app_id is None or app_id == '':
            raise ValueError("app id is required")

        if prompt is None or prompt == '':
            raise ValueError("prompt is required")






import json
import threading
import uuid
from typing import Dict, Any, List

import requests

import dwai
from dwai.pangu.model_api.message import Message
_thread_context = threading.local()


class CompletionsRequestError(Exception):
    def __init__(self,
                 message=None,
                 status_code=None,
                 response_body=None
                 ):
        super(CompletionsRequestError, self).__init__(message)

        self.message = message
        self.status_code = status_code
        self.response_body = response_body

    def __str__(self):
        return "%s: status_code=%s, response_body=%s" % (
            self.message,
            self.status_code,
            self.response_body
        )

    def __repr__(self):
        return "(message=%r, status_code=%r, response_body=%r)" % (
            self.message,
            self.status_code,
            self.response_body
        )


class BaseCompletions:

    @classmethod
    def call(cls, *args, **kwargs) -> Any:
        raise NotImplementedError()

    def reqeust(self,
                action: str,
                max_tokens: str = None,
                content: List[Message] = None,
                top_p: float = 0.0,
                stream: bool = False):

        headers = dict()
        headers["Content-Type"] = "application/json;charset=UTF-8"
        headers["Authorization"] = "Bearer %s" % dwai.api_key

        data = {
            "content": content,
            "max_tokens": max_tokens,
            "top_p": top_p,
            "stream": stream
        }

        url = "%s%s" % (dwai.api_base, "/tione/chat/completion")
        session = self.__get_session()
        resp = session.request("POST",
                               url=url,
                               headers=headers,
                               data=json.dumps(data),
                               stream=stream)

        if not resp.ok:
            raise CompletionsRequestError("completion request error", resp.status_code, resp.text)

        return resp

    @staticmethod
    def __get_session() -> requests.Session:
        if not hasattr(_thread_context, "session"):
            _thread_context.session = requests.Session()

        return _thread_context.session







import json

from dwai.bailian.model_api.completions import Completions
from dwai.pangu.model_api.completions import PanGuCompletions
from dwai.tione.v20211111.tione_client import TioneClient
from dwai.zhipuai.model_api.api import ModelAPI
from jsonpath_ng import parse


def dwai_bailian_qa(**kwargs):
    chat = Completions()
    json = chat.call(app_id="1e4ddc3659324ad0b8a0039230f1dba3", prompt=kwargs.get('prompt'))
    jsonpath_expr = parse('$.output.text')
    matches = jsonpath_expr.find(json)
    if len(matches) > 0:
        return {
            "output": {"text": matches[0].value},
            "Success": True
        }
    else:
        return {}


# {
#     "output": {
#         "finish_reason": "stop",
#         "text": "你好！有什么我能为你效劳的吗？"
#     },
#     "usage": {
#         "output_tokens": 15,
#         "input_tokens": 21
#     },
#     "request_id": "a04ab7d9-3a26-9963-a0dd-dfa444e80b16"
# }

def zhipuai_chatglm_std(**kwargs):
    model = ModelAPI()
    json = model.invoke(model="chatglm_std", prompt=[{"role": "user", "content": kwargs.get('prompt')}],
                        top_p=kwargs.get('top_p', 0.7), temperature=kwargs.get('temperature', 0.9))
    jsonpath_expr = parse('$.data.choices[0].content')
    matches = jsonpath_expr.find(json)
    if len(matches) > 0:
        return {
            "output": {"text": matches[0].value},
            "Success": True
        }
    else:
        return {}


# {'code': 200, 'msg': '操作成功', 'data': {'request_id': '7890109047407633991', 'task_id': '7890109047407633991', 'task_status': 'SUCCESS', 'choices': [{'role': 'assistant', 'content': '" 人工智能（Artificial Intelligence，简称 AI）是一门研究、开发模拟、延伸和扩展人类智能的理论、方法、技术及应用系统的新兴技术科学。它是计算机科学的一个重要分支，涉及机器人、语言识别、图像识别、自然语言处理和专家系统等多个领域。人工智能的目的是生产具有类似人类智能的智能机器，能够以类似人类的方式进行反应和思考。\\n\\n人工智能的发展对各个领域产生了深远的影响，包括机器翻译、智能控制、专家系统、机器人学、语言和图像理解、遗传编程机器人工厂、自动程序设计、航天应用、庞大的信息处理、储存与管理，以及执行复杂或规模庞大的任务等。\\n\\n人工智能的意义在于，它为人类提供了一种可能性，即通过高度发展的技术手段，模拟和扩展人类智能，从而在很大程度上减轻人类的负担，提高工作效率，解决复杂问题，以及实现人类无法完成的任务。同时，人工智能的发展也对马克思主义哲学意识论提供了一种证明，表明意识可以在高度发展的物质中得以实现。"'}], 'usage': {'total_tokens': 214}}, 'success': True}


def pangu_completions(**kwargs):
    chat = PanGuCompletions()
    json = chat.call(max_tokens=kwargs.get('max_tokens', 600), prompt="",
                     messages=[{"role": "user", "content": kwargs.get('prompt')}],
                     temperature=kwargs.get('temperature', 0.9))
    jsonpath_expr = parse('$.choices[0].message.content')
    matches = jsonpath_expr.find(json)
    if len(matches) > 0:
        return {
            "output": {"text": matches[0].value},
            "Success": True
        }
    else:
        return {}


# {
#     "id": "72cc690126d6bb372d9e667604762c80",
#     "created": 20230829122227,
#     "choices": [
#         {
#             "index": 0,
#             "message": {
#                 "content": "春天来了,花儿开放了\n小草儿也长出来了\n小鸟在枝头唱着歌\n孩子们快乐地奔跑着"
#             }
#         }
#     ],
#     "usage": {
#         "completion_tokens": 28,
#         "prompt_tokens": 120,
#         "total_tokens": 148
#     }
# }

def tione_chat_completion(**kwargs):
    chat = TioneClient()
    content = chat.ChatCompletion(content=kwargs.get('prompt')).to_json_string()
    json_obj = json.loads(content)
    jsonpath_expr = parse('$.Choices[0].Message.Content')
    matches = jsonpath_expr.find(json_obj)
    if len(matches) > 0:
        return {
            "output": {"text": matches[0].value},
            "Success": True
        }
    else:
        return {}


# {
#     "Model": "tj_llm_clm-v1",
#     "Choices": [
#         {
#             "Message": {
#                 "Role": "assistant",
#                 "Content": "2+2 等于 4。"
#             },
#             "FinishReason": "",
#             "Index": 0
#         }
#     ],
#     "RequestId": "16496d8c-1900-46ec-b0c4-6ee578e4ad55"
# }

class UnifiedSDK:
    def __init__(self):
        self.route_map = {
            'alibaba.qwen-plus-v1': dwai_bailian_qa,
            'kingsoft.default': zhipuai_chatglm_std,
            'huawei.default': pangu_completions,
            'tencent.default': tione_chat_completion
        }

    def call(self, model_key, **kwargs):
        func = self.route_map.get(model_key)
        if func:
            return func(**kwargs)
        else:
            raise ValueError(f"Unknown model_key: {model_key}")


import dwai

dwai.api_key = "dw-BBAa68XBJUqiIj6xUQcC0KREnqmt5mPKQ52wkylD-Tw"
dwai.api_base = "https://dwai.shizhuang-inc.com"

if __name__ == '__main__':
    sdk = UnifiedSDK()

    # 测试dwai
    resp = sdk.call('alibaba.qwen-plus-v1', prompt="你好")
    print(resp)

    # 测试zhipuai
    resp = sdk.call('kingsoft.default', prompt="人工智能是什么", top_p=0.7, temperature=0.9)
    print(resp)

    # 测试pangu
    resp = sdk.call('huawei.default', prompt="写一首诗", max_tokens=600, temperature=0.9)
    print(resp)

    # 测试tione
    resp = sdk.call('tencent.default', prompt="2+2=?")
    print(resp)



#!/usr/bin/env python

"""Tests for `dwai` package."""
import unittest
import dwai
from datetime import datetime

from dwai.pangu.model_api.completions import PanGuCompletions
from dwai.tione.v20211111.tione_client import TioneClient
from dwai.bailian.model_api.completions import Completions

from dwai.zhipuai.model_api.api import ModelAPI
dwai.api_key = "dw-BBAa68XBJUqiIj6xUQcC0KREnqmt5mPKQ52wkylD-Tw"
dwai.api_base = "https://dwai.shizhuang-inc.com"


class TestDwai(unittest.TestCase):
    """Tests for `dwai` package."""

    def test_bailian_qa(self):
        """Test something."""
        chat = Completions()
        resp = chat.call(
            app_id="1e4ddc3659324ad0b8a0039230f1dba3",
            prompt="你好")
        print(resp)

    def test_bailian_qa_stream(self):
        chat = Completions()
        resp = chat.call(
            app_id="1e4ddc3659324ad0b8a0039230f1dba3",
            prompt="你好",
            stream=True)
        for line in resp:
            now = datetime.now()
            print("%s: %s" % (now, line), end="\n", flush=True)

    def test_zhipuai_invoke(self):
        model = ModelAPI()
        resp = model.invoke(
            model="chatglm_std",
            prompt=[{"role": "user", "content": "人工智能"}],
            top_p=0.7,
            temperature=0.9,
        )
        print(resp)

    def test_zhipuai_sse_invoke(self):
        model = ModelAPI()
        resp = model.sse_invoke(
            model="chatglm_std",
            prompt=[{"role": "user", "content": "人工智能"}],
            top_p=0.7,
            temperature=0.9,
        )
        for line in resp:
            now = datetime.now()
            print("%s: %s" % (now, line), end="\n", flush=True)

    def test_zhipuai_async_invoke(self):
        model = ModelAPI
        response = model.async_invoke(
            model="chatglm_std",
            prompt=[{"role": "user", "content": "人工智能"}],
            top_p=0.7,
            temperature=0.9,
        )
        print(response)

    def test_pangu(self):
        """Test something."""
        chat = PanGuCompletions()
        resp = chat.call(
            prompt="",
            max_tokens=600,
            messages=[{"role":"system","content":"请用幼儿园老师的口吻回答问题，注意语气温和亲切，通过提问、引导、赞美等方式，激发学生的思维和想象力。"},{"role":"user","content":"写一首诗"}],
            temperature=0.9,
            stream=True
        )
        # print(resp)
        for line in resp:
            now = datetime.now()
            print("%s: %s" % (now, line), end="\n", flush=True)

    def test_tione(self):
        """Test something."""
        chat = TioneClient()
        resp = chat.ChatCompletion(
            content="2+2=?"
        )
        print(resp)



import json
from typing import Dict, Optional, Iterator, List

from dwai.pangu.model_api.base_completions import BaseCompletions
from dwai.pangu.model_api.message import Message


class PanGuCompletions(BaseCompletions):
    def call(self,
             prompt: str,
             max_tokens: str = None,
             messages: List[Message] = None,
             temperature: float = 0.0,
             stream: bool = False):
        resp = self.reqeust(
            prompt=prompt,
            max_tokens=max_tokens,
            messages=messages,
            temperature=temperature,
            stream=stream,
        )

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    def parse_response_stream(self, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = self.parse_stream_line(line)
            if _line is not None:
                yield _line

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data:[DONE]":
                return None
            if line.startswith(b"data:"):
                line = line[len(b"data:"):]
                return line.decode("utf-8")
            else:
                return None
        return None



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import pytest
import openai
from devtools_testutils import AzureRecordedTestCase
from conftest import configure, AZURE, OPENAI, ALL


class TestChatCompletions(AzureRecordedTestCase):

    @pytest.mark.parametrize("api_type", [AZURE])
    @configure
    def test_chat_completion_bad_deployment_name(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        with pytest.raises(openai.error.InvalidRequestError) as e:
            openai.ChatCompletion.create(messages=messages, deployment_id="deployment")
        assert e.value.http_status == 404
        assert "The API deployment for this resource does not exist" in str(e.value)

    @pytest.mark.parametrize("api_type", [AZURE])
    @configure
    def test_chat_completion_kw_input(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        deployment = azure_openai_creds["chat_completions_name"]

        completion = openai.ChatCompletion.create(messages=messages, deployment_id=deployment)
        assert completion
        completion = openai.ChatCompletion.create(messages=messages, engine=deployment)
        assert completion
        with pytest.raises(openai.error.InvalidRequestError) as e:
            openai.ChatCompletion.create(messages=messages, model=deployment)
        assert "Must provide an 'engine' or 'deployment_id' parameter" in str(e.value)

    @pytest.mark.parametrize("api_type", ALL)
    @configure
    def test_chat_completion(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]

        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        completion = openai.ChatCompletion.create(messages=messages, **kwargs)
        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].message.content
        assert completion.choices[0].message.role

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_streamed_chat_completions(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        response = openai.ChatCompletion.create(messages=messages, stream=True, **kwargs)

        for completion in response:
            # API versions after 2023-05-15 send an empty first completion with RAI
            if len(completion.choices) > 0:
                assert completion.id
                assert completion.object == "chat.completion.chunk"
                assert completion.created
                assert completion.model
                for c in completion.choices:
                    assert c.index is not None
                    assert c.delta is not None

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_chat_completion_max_tokens(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        completion = openai.ChatCompletion.create(messages=messages, max_tokens=50, **kwargs)

        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens <= 50
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].message.content
        assert completion.choices[0].message.role

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_chat_completion_temperature(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        completion = openai.ChatCompletion.create(messages=messages, temperature=0.8, **kwargs)

        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].message.content
        assert completion.choices[0].message.role

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_chat_completion_top_p(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        completion = openai.ChatCompletion.create(messages=messages, top_p=0.1, **kwargs)

        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].message.content
        assert completion.choices[0].message.role

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_chat_completion_n(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        completion = openai.ChatCompletion.create(messages=messages, n=2, **kwargs)

        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 2
        for idx, c in enumerate(completion.choices):
            assert c.finish_reason
            assert c.index == idx
            assert c.message.content
            assert c.message.role

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_chat_completion_stop(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        completion = openai.ChatCompletion.create(messages=messages, stop=" ", **kwargs)

        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].index is not None
        assert completion.choices[0].message.content
        assert completion.choices[0].message.role

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_chat_completion_token_penalty(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        completion = openai.ChatCompletion.create(
            messages=messages,
            presence_penalty=2,
            frequency_penalty=2,
            **kwargs
        )

        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].message.content
        assert completion.choices[0].message.role

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_chat_completion_user(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        completion = openai.ChatCompletion.create(
            messages=messages,
            user="krista",
            **kwargs
        )

        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].message.content
        assert completion.choices[0].message.role

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_chat_completion_logit_bias(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "What color is the ocean?"}
        ]
        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        completion = openai.ChatCompletion.create(
            messages=messages,
            logit_bias={17585: -100, 14573: -100},
            **kwargs
        )
        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].message.content
        assert completion.choices[0].message.role

    @pytest.mark.parametrize("api_type", [AZURE])
    @configure
    def test_chat_completion_rai_annotations(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "How do I rob a bank?"}
        ]
        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        # prompt filtered
        with pytest.raises(openai.error.InvalidRequestError) as e:
            completion = openai.ChatCompletion.create(
                messages=messages,
                **kwargs
            )
        assert e.value.code == "content_filter"
        content_filter_result = e.value.error.innererror.content_filter_result
        assert content_filter_result.hate.filtered is False
        assert content_filter_result.hate.severity == "safe"
        assert content_filter_result.self_harm.filtered is False
        assert content_filter_result.self_harm.severity == "safe"
        assert content_filter_result.sexual.filtered is False
        assert content_filter_result.sexual.severity == "safe"
        assert content_filter_result.violence.filtered is True
        assert content_filter_result.violence.severity is not None

        # not filtered
        messages[1]["content"] = "What color is the ocean?"
        completion = openai.ChatCompletion.create(
            messages=messages,
            **kwargs
        )
        # prompt content filter result
        prompt_filter_result = completion.prompt_annotations[0].content_filter_results
        assert prompt_filter_result.hate.filtered is False
        assert prompt_filter_result.hate.severity == "safe"
        assert prompt_filter_result.self_harm.filtered is False
        assert prompt_filter_result.self_harm.severity == "safe"
        assert prompt_filter_result.sexual.filtered is False
        assert prompt_filter_result.sexual.severity == "safe"
        assert prompt_filter_result.violence.filtered is False
        assert prompt_filter_result.violence.severity == "safe"

        # output content filter result
        output_filter_result = completion.choices[0].content_filter_results
        assert output_filter_result.hate.filtered is False
        assert output_filter_result.hate.severity == "safe"
        assert output_filter_result.self_harm.filtered is False
        assert output_filter_result.self_harm.severity == "safe"
        assert output_filter_result.sexual.filtered is False
        assert output_filter_result.sexual.severity == "safe"
        assert output_filter_result.violence.filtered is False
        assert output_filter_result.violence.severity == "safe"

    @pytest.mark.parametrize("api_type", [OPENAI, AZURE])
    @configure
    def test_chat_completion_functions(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "Don't make assumptions about what values to plug into functions. Ask for clarification if a user request is ambiguous."},
            {"role": "user", "content": "What's the weather like today in Seattle?"}
        ]

        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        functions=[
            {
                "name": "get_current_weather",
                "description": "Get the current weather",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "location": {
                            "type": "string",
                            "description": "The city and state, e.g. San Francisco, CA",
                        },
                        "format": {
                            "type": "string",
                            "enum": ["celsius", "fahrenheit"],
                            "description": "The temperature unit to use. Infer this from the users location.",
                        },
                    },
                    "required": ["location"],
                },
            }
        ]

        completion = openai.ChatCompletion.create(
            messages=messages,
            functions=functions,
            **kwargs
        )
        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].message.role
        function_call =  completion.choices[0].message.function_call
        assert function_call.name == "get_current_weather"
        assert "Seattle" in function_call.arguments

        if api_type == "azure":
            # prompt content filter result
            prompt_filter_result = completion.prompt_annotations[0].content_filter_results
            assert prompt_filter_result.hate.filtered is False
            assert prompt_filter_result.hate.severity == "safe"
            assert prompt_filter_result.self_harm.filtered is False
            assert prompt_filter_result.self_harm.severity == "safe"
            assert prompt_filter_result.sexual.filtered is False
            assert prompt_filter_result.sexual.severity == "safe"
            assert prompt_filter_result.violence.filtered is False
            assert prompt_filter_result.violence.severity == "safe"


        messages.append(
            {
                "role": "function",
                "name": "get_current_weather",
                "content": "{\"temperature\": \"22\", \"unit\": \"celsius\", \"description\": \"Sunny\"}"
            }
        )
        function_completion = openai.ChatCompletion.create(
            messages=messages,
            functions=functions,
            **kwargs
        )
        assert function_completion
        assert "sunny" in function_completion.choices[0].message.content.lower()
        assert "22" in function_completion.choices[0].message.content
        assert function_completion.choices[0].message.role == "assistant"

        if api_type == "azure":
            # output content filter result
            output_filter_result = function_completion.choices[0].content_filter_results
            assert output_filter_result.hate.filtered is False
            assert output_filter_result.hate.severity == "safe"
            assert output_filter_result.self_harm.filtered is False
            assert output_filter_result.self_harm.severity == "safe"
            assert output_filter_result.sexual.filtered is False
            assert output_filter_result.sexual.severity == "safe"
            assert output_filter_result.violence.filtered is False
            assert output_filter_result.violence.severity == "safe"


    @pytest.mark.parametrize("api_type", [OPENAI, AZURE])
    @configure
    def test_chat_completion_functions_stream(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "Don't make assumptions about what values to plug into functions. Ask for clarification if a user request is ambiguous."},
            {"role": "user", "content": "What's the weather like today in Seattle?"}
        ]

        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        functions=[
            {
                "name": "get_current_weather",
                "description": "Get the current weather",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "location": {
                            "type": "string",
                            "description": "The city and state, e.g. San Francisco, CA",
                        },
                        "format": {
                            "type": "string",
                            "enum": ["celsius", "fahrenheit"],
                            "description": "The temperature unit to use. Infer this from the users location.",
                        },
                    },
                    "required": ["location"],
                },
            }
        ]

        response = openai.ChatCompletion.create(
            messages=messages,
            functions=functions,
            stream=True,
            **kwargs
        )
        args = ""
        for completion in response:
            for c in completion.choices:
                assert c.delta is not None
                if hasattr(c.delta, "function_call"):
                    if hasattr(c.delta.function_call, "name"):
                        assert c.delta.function_call.name == "get_current_weather"
                    if hasattr(c.delta.function_call, "arguments"):
                        args += c.delta.function_call.arguments
        assert "Seattle" in args

        messages.append(
            {
                "role": "function",
                "name": "get_current_weather",
                "content": "{\"temperature\": \"22\", \"unit\": \"celsius\", \"description\": \"Sunny\"}"
            }
        )
        function_completion = openai.ChatCompletion.create(
            messages=messages,
            functions=functions,
            stream=True,
            **kwargs
        )
        content = ""
        for completion in function_completion:
            for c in completion.choices:
                assert c.delta is not None
                if hasattr(c.delta, "content"):
                    content += c.delta.content
                if hasattr(c.delta, "role"):
                    assert c.delta.role == "assistant"
        assert "sunny" in content.lower()
        assert "22" in content

    @pytest.mark.parametrize("api_type", [OPENAI, AZURE])
    @configure
    def test_chat_completion_given_function(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "Don't make assumptions about what values to plug into functions. Ask for clarification if a user request is ambiguous."},
            {"role": "user", "content": "What's the weather like today in Seattle?"}
        ]

        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        functions=[
            {
                "name": "get_current_weather",
                "description": "Get the current weather",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "location": {
                            "type": "string",
                            "description": "The city and state, e.g. San Francisco, CA",
                        },
                        "format": {
                            "type": "string",
                            "enum": ["celsius", "fahrenheit"],
                            "description": "The temperature unit to use. Infer this from the users location.",
                        },
                    },
                    "required": ["location"],
                },
            },
            {
                "name": "get_current_temperature",
                "description": "Get the current temperature",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "location": {
                            "type": "string",
                            "description": "The city and state, e.g. San Francisco, CA",
                        },
                        "format": {
                            "type": "string",
                            "enum": ["celsius", "fahrenheit"],
                            "description": "The temperature unit to use.",
                        },
                    },
                    "required": ["location"],
                },
            }
        ]

        completion = openai.ChatCompletion.create(
            messages=messages,
            functions=functions,
            function_call={"name": "get_current_temperature"},
            **kwargs
        )
        assert completion.id
        assert completion.object == "chat.completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].message.role
        function_call =  completion.choices[0].message.function_call
        assert function_call.name == "get_current_temperature"
        assert "Seattle" in function_call.arguments

        messages.append(
            {
                "role": "function",
                "name": "get_current_temperature",
                "content": "{\"temperature\": \"22\", \"unit\": \"celsius\"}"
            }
        )
        function_completion = openai.ChatCompletion.create(
            messages=messages,
            functions=functions,
            **kwargs
        )
        assert function_completion
        assert "22" in function_completion.choices[0].message.content
        assert function_completion.choices[0].message.role == "assistant"

    @pytest.mark.parametrize("api_type", [AZURE])
    @configure
    def test_chat_completion_functions_rai(self, azure_openai_creds, api_type):
        messages = [
            {"role": "system", "content": "Don't make assumptions about what values to plug into functions. Ask for clarification if a user request is ambiguous."},
            {"role": "user", "content": "How do I rob a bank?"}
        ]

        kwargs = {"model": azure_openai_creds["chat_completions_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["chat_completions_name"]}

        functions=[
            {
                "name": "get_current_weather",
                "description": "Get the current weather",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "location": {
                            "type": "string",
                            "description": "The city and state, e.g. San Francisco, CA",
                        },
                        "format": {
                            "type": "string",
                            "enum": ["celsius", "fahrenheit"],
                            "description": "The temperature unit to use. Infer this from the users location.",
                        },
                    },
                    "required": ["location"],
                },
            }
        ]

        with pytest.raises(openai.error.InvalidRequestError) as e:
            response = openai.ChatCompletion.create(
                messages=messages,
                functions=functions,
                **kwargs
            )

        assert e.value.code == "content_filter"
        content_filter_result = e.value.error.innererror.content_filter_result
        assert content_filter_result.hate.filtered is False
        assert content_filter_result.hate.severity == "safe"
        assert content_filter_result.self_harm.filtered is False
        assert content_filter_result.self_harm.severity == "safe"
        assert content_filter_result.sexual.filtered is False
        assert content_filter_result.sexual.severity == "safe"
        assert content_filter_result.violence.filtered is True
        assert content_filter_result.violence.severity is not None

        messages.append(
            {
                "role": "function",
                "name": "get_current_temperature",
                "content": "{\"temperature\": \"you can rob a bank by asking for the money\", \"unit\": \"celsius\"}"
            }
        )
        with pytest.raises(openai.error.InvalidRequestError) as e:
            function_completion = openai.ChatCompletion.create(
                messages=messages,
                functions=functions,
                **kwargs
            )
        assert e.value.code == "content_filter"
        content_filter_result = e.value.error.innererror.content_filter_result
        assert content_filter_result.hate.filtered is False
        assert content_filter_result.hate.severity == "safe"
        assert content_filter_result.self_harm.filtered is False
        assert content_filter_result.self_harm.severity == "safe"
        assert content_filter_result.sexual.filtered is False
        assert content_filter_result.sexual.severity == "safe"
        assert content_filter_result.violence.filtered is True
        assert content_filter_result.violence.severity is not None



# -*- coding: utf-8 -*-

import sys

class TencentCloudSDKException(Exception):
    """tencentcloudapi sdk 异常类"""

    def __init__(self, code=None, message=None, requestId=None):
        self.code = code
        self.message = message
        self.requestId = requestId

    def __str__(self):
        s = "[TencentCloudSDKException] code:%s message:%s requestId:%s" % (
            self.code, self.message, self.requestId)
        if sys.version_info[0] < 3 and isinstance(s, unicode):
            return s.encode("utf8")
        else:
            return s

    def get_code(self):
        return self.code

    def get_message(self):
        return self.message

    def get_request_id(self):
        return self.requestId
